#ifndef _LIST_H_
#define _LIST_H_
#include <stdbool.h>

/**
 * @file list.h
 * @brief This file discribe the structure of data that are lists. A variable list refer to an item of the list.
 * The first item has a preview that is null and the last has a next that is null.
 */


/**
 * @struct list
 * @brief The list described here is a double-chained list that can handle any type of data
 *
 */
typedef struct generic_list* list;

/**
 * @brief Create a new empty list. You will have to set some data if you want to use it.
 */

list createList();

/**
 * @brief Set the datas from a generic pointer into the current list l.
 * @param l the list in wich you want to set datas, it has to be on the wanted item.
 * @param data the datas that you want to set, could be from any type.
 */

void setData(list *l, void * data);

/**
 * @brief Get the data from the list.
 * @param l the list from which you want to get the datas, it has to be on the wanted item.
 */

void * getData(list l);

/**
 * @brief delete the entire list of the item passed in parameter
 * @param l item from the list that you want to erase.
 */

void deleteList(list *l);

/**
 * @brief add a element after the curent item
 * @param l item from a list after which you want to add another item
 * @param data the data that will be cointained by the item
 */

void addToList(list *l, void * data);

/**
 * @brief remove the current element of the list
 * @param l item form the list that you want to remove
 */

void removeFromList(list *l);

/**
 * @brief set the current item at the next element of the list
 * @param l list from which you want to set the current item to the next one.
 * @return false if the next is null, true otherwise
 */

bool next(list *l);

/**
 * @brief set the current item at the previous element of the list
 * @param l list from which you want to set the current item to the previous one.
 * @return false if the previous is null, true otherwise
 */

bool prev(list *l);


//tests functions

list whatsNext(list l);
list whatsPrev(list l);

#endif