#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>
#include <string.h>

#include <game.h>

#define SIZE_SQUARE 3
#define MAX_INPUT_SIZE 25



void remplir_grille(int** array, cgame g)
{
    int nb_pieces = game_nb_pieces(g);
    int width = game_width(g), height = game_height(g);

    for (int i = 0; i < height; i++){
        for (int j = 0; j < width; j++)
        {
            array[i][j] = -1;
		}
	}

    for (int k = 0; k < nb_pieces; k++) {
        int x = get_x(game_piece(g, k));
        int y = get_y(game_piece(g, k));
        for (int i = 0; i < get_width(game_piece(g, k)); ++i)
            for (int j = 0; j < get_height(game_piece(g, k)); ++j)
                array[((height-1)-y)-j][x+i] = k;
    }
}


void afficher_game(int** array, cgame g)
{
	
	remplir_grille(array, g);
	
    int board_width = game_width(g), board_height = game_height(g);
    
    char* dashes = "-------";
    for (int j = 0; j < board_height; j++) {
        printf("*");
        for (int l = 0; l <board_width ; l++){
            printf("%s*", dashes);
		}
        printf("\n");
        for (int k = 0; k < SIZE_SQUARE; k++) {
            printf("*");
            for (int i = 0; i < board_width; i++) {
                if(k == (SIZE_SQUARE/2))
                {
                    int piece_tag = array[j][i];
                    if(piece_tag < 0)
                        printf("       *");
                    else if(piece_tag < 10)
                        printf("   %d   *", piece_tag);
                    else
                        printf("   %d  *", piece_tag);

                }
                else
                    printf("       *");
            }
            printf("\n");
        }
    }
    printf("*");
    for (int i = 0; i < board_width; ++i) {
        if(i == 1 || i == 2)
            printf("       *");
        else
            printf("%s*", dashes);
    }
    printf("\n");
}



piece* prepare_game(int * nb_pieces, int difficulty)
{
    piece * liste_piece;
    if(difficulty == 1)
    {
        *nb_pieces = 3;
        liste_piece = malloc(*nb_pieces * sizeof(piece));
        assert(liste_piece != NULL);
        liste_piece[0] = new_piece(1, 3, 2, 2 ,true, true);
        liste_piece[1] = new_piece(1, 1, 2, 2,true, true);
        liste_piece[2] = new_piece(1, 0, 2, 1,true, true);
    }
    else if(difficulty == 2)
    {
        *nb_pieces = 10;
        liste_piece = malloc(*nb_pieces * sizeof(piece));
        assert(liste_piece != NULL);
        liste_piece[0] = new_piece(1, 3, 2, 2,true, true);
        liste_piece[1] = new_piece(0, 0, 1, 1,true, true);
        liste_piece[2] = new_piece(0, 1, 1, 2,true, true);
        liste_piece[3] = new_piece(0, 3, 1, 2,true, true);
        liste_piece[4] = new_piece(1, 1, 1, 1,true, true);
        liste_piece[5] = new_piece(1, 2, 2, 1,true, true);
        liste_piece[6] = new_piece(2, 1, 1, 1,true, true);
        liste_piece[7] = new_piece(3, 0, 1, 1,true, true);
        liste_piece[8] = new_piece(3, 1, 1, 2,true, true);
        liste_piece[9] = new_piece(3, 3, 1, 2,true, true);
    }
    return liste_piece;
}

bool game_over_ar(cgame g)
{
    return game_over(g,1,0);
}

int main(int argc, char * argv [])
{
    int difficulty;
    
    if (argc != 2 || !(difficulty = atoi (argv[1])))
    {
        fprintf(stderr,"Usage : ane_rouge [difficulte]\n[difficulte] = 1 ou 2.\n");
        return EXIT_FAILURE;
    }
    
    if(difficulty < 1 || difficulty > 2)
        difficulty = 1;
    
    int nb_pieces = 0;
    piece * liste_piece= prepare_game(&nb_pieces, difficulty);
   
   
    game g = new_game(4, 5, nb_pieces, liste_piece);
    
      
    int ** array = (int**)malloc(game_height(g)* sizeof(int*));
    for (int i = 0; i < game_height(g); ++i)
    {
		array[i] = malloc(game_width(g) * sizeof(int));
	}
	
    afficher_game(array, g);


    int numero_piece, distance;
    char direction_saisie[MAX_INPUT_SIZE];
    char piece_saisie[MAX_INPUT_SIZE];
    char distance_saisie[MAX_INPUT_SIZE];
    dir direction_reele;


    while(!game_over_ar(g))
    {
        printf("Veuillez saisir le numéro de la pièce à déplacer (saisir un entier):\n");
        scanf ("%s", piece_saisie);
        numero_piece = atoi(piece_saisie);
        if(numero_piece < 0 || numero_piece >= nb_pieces)
        {
            printf("\nVOUS DEVEZ CHOISIR UN NUMERO DE PIECE ENTRE %d ET %d\n\n", 0, nb_pieces-1);
            continue;
        }
        printf ("Veuillez saisir la direction du déplacement (RIGHT, LEFT, UP ou DOWN) :\n");
        scanf ("%s", direction_saisie);
        if (!strcmp (direction_saisie, "RIGHT") || !strcmp (direction_saisie, "right"))
            direction_reele = RIGHT;
        else if (!strcmp (direction_saisie, "LEFT") || !strcmp (direction_saisie, "left"))
            direction_reele = LEFT;
        else if (!strcmp (direction_saisie, "UP") || !strcmp (direction_saisie, "up"))
            direction_reele = UP;
        else if (!strcmp (direction_saisie, "DOWN") || !strcmp (direction_saisie, "down"))
            direction_reele = DOWN;
        else
        {
            printf ("\nDIRECTION SAISIE INCORRECTE !\n\n");
            continue;
        }
        printf("Veuillez saisir la distance du déplacement (saisir un entier):\n");
        scanf ("%s", distance_saisie);
        distance = atoi(distance_saisie);
        if (!play_move (g, numero_piece, direction_reele, distance))
        {
            printf("\nVOUS NE POUVEZ PAS EFFECTUER CE MOUVEMENT !\n\n");
            continue;
        }
        afficher_game(array, g);
    }

    printf ("\n\n\n\n\n\n\n\n VOUS AVEZ GAGNE !!! \n VOTRE SCORE : %d COUPS\n\n\n\n\n\n\n",game_nb_moves(g));


    delete_game(g);
    
    for (int i = 0; i < game_height(g); ++i){
		free(array[i]);
	}
	free(array);


    for (int i = 0; i < nb_pieces; ++i){
        delete_piece(liste_piece[i]);
	}
    free(liste_piece);

    return EXIT_SUCCESS;
}
