#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <game.h>

#define RUSH_HOUR_SIZE 6
/**
 * nb_pieces correspond au nombre de pièces dans la partie.
 * liste_piece correspond aux pièces qui seront dans la partie.
 * mouvement correspond au nombre de mouvements qui ont étaient réalisés dans la partie.
 */

struct game_s
{
    int nb_pieces;
    piece *liste_piece;
    int mouvement;
    int width;
    int height;
};

/**
 * Créer une nouvelle partie avec nb_pieces pièces.
 * Ces pièces sont celles du tableau pieces.
 **/

game new_game_hr (int nb_pieces, piece * pieces)
{
    game newgame = new_game(RUSH_HOUR_SIZE,RUSH_HOUR_SIZE, nb_pieces, pieces);
    return newgame;
}

/**
 * Supprime la partie g et libère l'espace mémoire alloué.
//*/

void delete_game (game g)
{
    for (int i = 0; i < g->nb_pieces; i++)
    {
        delete_piece(g->liste_piece[i]);
    }
    free (g->liste_piece);
    free (g);
}

/**
 * Copie la partie src dans la partie dst.
 **/

void copy_game (cgame src, game dst)
{
    for(int i=0; i< dst->nb_pieces; i++){
        delete_piece(dst->liste_piece[i]);
    }
    dst->liste_piece = realloc(dst->liste_piece,(src->nb_pieces * (sizeof (piece))));
    for (int i=0; i < src->nb_pieces; i++){
		dst->liste_piece[i]=new_piece(get_x(src->liste_piece[i]),get_y(src->liste_piece[i]),
		get_width(src->liste_piece[i]),get_height(src->liste_piece[i]),
		can_move_x(src->liste_piece[i]),can_move_y(src->liste_piece[i]));
    }
    dst->nb_pieces = src->nb_pieces;
    dst->mouvement = src->mouvement;
    dst->width = src->width;
    dst->height = src->height;
}


/**
 * Renvoie le nombre de pièces dans la partie g.
 **/

int game_nb_pieces (cgame g)
{
    return g->nb_pieces;
}


/**
 * Renvoie la pièce donc l'indice dans le tableau de pièces est piece_num.
 **/


cpiece game_piece (cgame g, int piece_num)
{
    return g->liste_piece[piece_num];
}



bool game_over_hr (cgame g)
{
    return game_over (g, 4, 3);
}

/**
 * Effectue le mouvement de la pièce d'indice piece_num dans la direction d sur une distance distance si possible,
 * en tenant compte des collisions. Retourne vrai si le déplacement complet a pu être réalisé.
 **/


bool play_move(game g, int piece_num, dir d, int distance){
    piece p = (g->liste_piece[piece_num]);
    if((!can_move_y(p) && (d == UP || d == DOWN)) || (!can_move_x(p) && (d == RIGHT || d == LEFT))){
        return false;
    }
    for(int i = 0; i < distance; i++)
    {
		move_piece(p, d, 1);
        if(d == RIGHT || d == LEFT)
        {
            if(get_x(p)>(game_width(g)-get_width(p)) || get_x(p)<0)
            {
				for(int j=0; j<=i; j++){
					move_piece(p,d,-1);
			}
                return false;
            }
        }
        else
        {
            if(get_y(p)>game_height(g) || get_y(p)<0)
            {
                for(int j=0; j<=i; j++){
					move_piece(p,d,-1);
			}
                return false;
            }
        }
        for (int k = 0; k < g->nb_pieces; k++)
        {
            if (k != piece_num)
            {
                if (intersect(p, g->liste_piece[k]))
                {
					for(int j=0; j<=i; j++){
					move_piece(p,d,-1);
			}
                    return false;
				}
			}
		}
	}
    g->mouvement ++;
    return true;
}


/**
 * Renvoie le nombre de mouvements qui ont étaient effectuer dans la partie
 **/


int game_nb_moves (cgame g)
{
    return g->mouvement;
}

//////////////////////////////// Version 2 /////////////////////////////////////// 

game new_game (int width, int height, int nb_pieces, piece * pieces)
{
    game newgame = malloc (sizeof (struct game_s));
    newgame->nb_pieces = nb_pieces;
    newgame->liste_piece = malloc (nb_pieces * (sizeof (piece)));
    for (int i = 0; i < nb_pieces; i++)
    {
        newgame->liste_piece[i] =
                new_piece (get_x (pieces[i]), get_y (pieces[i]),
                           get_width (pieces[i]), get_height (pieces[i]),
                           can_move_x (pieces[i]), can_move_y (pieces[i]));
    }
    newgame->mouvement = 0;
    newgame->width = width;
    newgame->height = height;
    return newgame;
}



int
game_width (cgame g)
{
    return g->width;
}


int
game_height (cgame g)
{
    return g->height;
}


int
game_square_piece (game g, int x, int y)
{
    piece case_a_tester = new_piece (x, y, 1, 1, false, false);
    for (int i = 0; i < g->nb_pieces; i++)
    {
        if (intersect (g->liste_piece[i], case_a_tester))
        {
			return i;
        }
    }
    return -1;
}


/**
 * Indique si la partie est terminée.
 **/

bool game_over (cgame g, int posX, int posY)
{
    return get_x(g->liste_piece[0]) == posX
           && get_y (g->liste_piece[0]) == posY;
}
