#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>

#include <piece.h>

/**
 * x,y sont les coordonnées du point bas-gauche de la pièce.
 * small correspond à la taille de la pièce, si small==true la pièce a une taille de 2, si small==false la pièce a une taille de 3.
 * horizontal indique la direction de la pièce, si horizontal==true la pièce peut se déplacer horizontalement, si horizontal==false la pièce peut se déplacer verticalement.
 * tag indique le numéro de la pièce.
 */

/** NB_PIECE permet de savoir le nombre de pièce présentes dans la partie
 * et permettra de définir le tag de la prochaine pièce à créer
 */
struct piece_s
{
    int x;
    int y;
    int width;
    int height;
    bool move_x;
    bool move_y;
};

piece new_piece_rh (int x, int y, bool small, bool horizontal)
{
    int width, height;
    if (horizontal)
    {
        height = 1;
        if (small)
            width = 2;
        else
            width = 3;
    }
    else
    {
        width = 1;
        if (small)
            height = 2;
        else
            height = 3;
    }
    return new_piece(x, y, width, height, horizontal, (!horizontal));
}


/**
 * Créer une nouvelle pièce dont les coordonnées de la case bas gauche sont x,y.
 * Sa taille est de 2 cases si small est vrai, de 3 cases sinon.
 * Cette pièce est horizontale si horizontal est vrai, verticale sinon.
*/

piece new_piece (int x, int y, int width, int height, bool move_x, bool move_y)
{
    piece newpiece = malloc (sizeof (struct piece_s));
    newpiece->x = x;
    newpiece->y = y;
    newpiece->width = width;
    newpiece->height = height;
    newpiece->move_x = move_x;
    newpiece->move_y = move_y;
    return newpiece;
}


/**
 * Supprime la pièce p et libère la mémoire allouée.
*/
void
delete_piece (piece p)
{
    free (p);
}

/**
 * Copie la pièce src dans la pièce dst
 */
void
copy_piece (cpiece src, piece dst)
{
    dst->x = src->x;
    dst->y = src->y;
    dst->width = src->width;
    dst->height = src->height;
    dst->move_x = src->move_x;
    dst->move_y = src->move_y;
}

/**
 * Déplace la pièce p d'un nombre de case égal à distance dans la direction indiquée par d.
 * Si le déplacement voulu est UP ou DOWN, alors la pièce est déplacée si elle est verticale, il ne se passe rien sinon.
 * si le déplacement voulu est LEFT ou RIGHT, alors la pièce est déplacée si elle est horizontale, il ne se passe rien sinon.
 */
void
move_piece (piece p, dir d, int distance)
{
    switch (d)
    {
        case 0:			// déplacement UP voulu
            if (can_move_y (p))
            {
                p->y += distance;
            };
            break;
        case 1:			// déplacement LEFT voulu
            if (can_move_x (p))
            {
                p->x -= distance;
            };
            break;
        case 2:			// déplacement DOWN voulu
            if (can_move_y (p))
            {
                p->y -= distance;
            };
            break;
        case 3:			// déplacement RIGHT voulu
            if (can_move_x (p))
            {
                p->x += distance;
            };
            break;
    }
}


/**
*test si a chaque case de la piece p1, une case de la piece p2 est en commun
**/
bool intersect (cpiece p1, cpiece p2)
{
    for (int i =p1->x; i <(p1->x+ p1->width); i++)
    {
        for (int j = p1->y; j <(p1->y + p1->height); j++)
        {
            for (int y = p2->x; y <(p2->x+ p2->width); y++)
            {
                for (int x = p2->y; x <(p2->y + p2->height); x++)
                {
                    if (i == y && j == x)
                    {
                        return true;
					}
                }
            }
        }
    }
    return false;
}


/**
*retourne l'abscisse le plus à gauche
**/
int
get_x (cpiece p)
{
    return p->x;
}


/**
*retourne l'ordonné le plus bas
**/
int
get_y (cpiece p)
{
    return p->y;
}


/**
*retourne la hauteur de la piece
**/
int
get_height (cpiece p)
{
    return p->height;
}


/**
*retourne la largeur de la piece
**/
int
get_width (cpiece p)
{
    return p->width;
}


/**
 *retourne true si horizontal, false sinon
 **/
bool is_horizontal (cpiece p)
{
    if (p->width > p->height)
        return true;
    return false;
}


/**
 *true si la piece peut se déplacer sur x, false sinon
 **/
bool can_move_x (cpiece p)
{
    return p->move_x;
}


/**
*true si la piece peut se deplacersur y, false sinon
**/
bool can_move_y (cpiece p)
{
    return p->move_y;
}

 
