#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include <game.h>

#define PIECES_MAX 18
#define MAX_INPUT_SIZE 25


bool
isCaseOccupee (cgame g, int j, int k, int array[6][6])
{
    return (array[j][k] >= 0 && array[j][k] <= game_nb_pieces (g));
}

void remplir_grille(int array[6][6], cgame g)
{
    int nb_pieces = game_nb_pieces(g);
    int width = 6, height = 6;

    for (int i = 0; i < height; i++)
        for (int j = 0; j < width; j++)
            array[i][j] = -1;

    for (int k = 0; k < nb_pieces; k++) {
        int x = get_x(game_piece(g, k));
        int y = get_y(game_piece(g, k));
        for (int i = 0; i < get_width(game_piece(g, k)); ++i)
            for (int j = 0; j < get_height(game_piece(g, k)); ++j)
                array[(5-y)-j][x+i] = k;
    }
}


void
afficher_game (cgame g, int array[6][6])
{
    remplir_grille(array, g);
    printf ("*-----*-----*-----*-----*-----*-----*\n");
    for (int j = 0; j < 6; j++)
    {
        for (int k = 0; k < 6; k++)
        {
            printf ("|");
            if (isCaseOccupee (g, j, k, array))
            {
                printf ("  %d  ", array[j][k]);
            }
            else
            {
                printf ("     ");
            }
        }
        printf ("|");
        if (j == 2)
        {
            printf ("==> EXIT");
        }
        printf ("\n");
        printf ("*-----*-----*-----*-----*-----*-----*\n");
    }
}


int
preparer_game (piece * p, int difficulty)
{
    if (difficulty == 1)
    {
        p[0] = new_piece_rh (0, 3, true, true);
        p[1] = new_piece_rh (3, 0, false, true);
        p[2] = new_piece_rh (2, 2, false, false);
        return 3;

    }
    else if (difficulty == 2)
    {
        p[0] = new_piece_rh (0, 3, true, true);
        p[1] = new_piece_rh (4, 1, true, true);
        p[2] = new_piece_rh (2, 1, false, false);
        p[3] = new_piece_rh (3, 5, false, true);
        p[4] = new_piece_rh (4, 3, true, false);
        p[5] = new_piece_rh (1, 0, true, true);
        return 6;
    }
    else if (difficulty == 3)
    {
        p[0] = new_piece_rh (0, 3, true, true);
        p[1] = new_piece_rh (1, 0, true, true);
        p[2] = new_piece_rh (5, 1, false, false);
        p[3] = new_piece_rh (2, 1, false, false);
        p[4] = new_piece_rh (4, 2, true, false);
        p[5] = new_piece_rh (3, 5, true, true);
        p[6] = new_piece_rh (3, 0, false, true);
        p[7] = new_piece_rh (0, 0, true, false);
        p[8] = new_piece_rh (1, 5, true, true);
        return 9;
    }
    return -1;
}



int
main (int argc, const char *argv[])
{
    int difficulty;
    
    if (argc != 2 || !(difficulty = atoi (argv[1])))
    {
        fprintf(stderr,"Usage : rush_hour. Entrez en argument difficulté entre 1 et 3; sinon par défaut difficulte = 1\n");
        return EXIT_FAILURE;
    }
    if (difficulty < 1 || difficulty > 3)
    {
        difficulty = 1;
    }


    piece liste_piece[PIECES_MAX];
    int nb_pieces=preparer_game (liste_piece, difficulty);


    int array[6][6];
    game g = new_game_hr (nb_pieces, liste_piece);
    afficher_game (g, array);

    int numero_piece, distance;
    char direction_saisie[MAX_INPUT_SIZE];
    char piece_saisie[MAX_INPUT_SIZE];
    char distance_saisie[MAX_INPUT_SIZE];
    dir direction_reele;

    while (!game_over_hr (g))
    {
        printf("Veuillez saisir le numéro de la pièce à déplacer (saisir un entier):\n");
        scanf ("%s", piece_saisie);
        numero_piece = atoi(piece_saisie);
        if (numero_piece < 0 || numero_piece > nb_pieces-1)
        {
            printf
                    ("Vous devez saisir un numéro de pièce entre %d et %d !\n\n", 0,
                     nb_pieces-1);
            continue;
        }
        printf ("Veuillez saisir la direction du déplacement (RIGHT, LEFT, UP ou DOWN) :\n");
        scanf ("%s", direction_saisie);
        if (!strcmp (direction_saisie, "RIGHT") || !strcmp (direction_saisie, "right"))
            direction_reele = RIGHT;
        else if (!strcmp (direction_saisie, "LEFT") || !strcmp (direction_saisie, "left"))
            direction_reele = LEFT;
        else if (!strcmp (direction_saisie, "UP") || !strcmp (direction_saisie, "up"))
            direction_reele = UP;
        else if (!strcmp (direction_saisie, "DOWN") || !strcmp (direction_saisie, "down"))
            direction_reele = DOWN;
        else
        {
            printf ("Direction saisie incorrecte !\n\n");
            continue;
        }
        printf("Veuillez saisir la distance du déplacement (saisir un entier):\n");
        scanf ("%s", distance_saisie);
        distance = atoi(distance_saisie);
        if (!play_move (g, numero_piece, direction_reele, distance))
        {
            printf("Vous ne pouvez pas effectuer ce mouvement !\n\n");
            continue;
        }
        afficher_game (g, array);
    }
    printf ("\n\n\n\n\n\n\n\n VOUS AVEZ GAGNE !!! \n VOTRE SCORE : %d COUPS\n\n\n\n\n\n\n",game_nb_moves(g));
    
    
    delete_game(g);

    for (int i = 0; i < nb_pieces; i++){
		delete_piece(liste_piece[i]);
	}

    return EXIT_SUCCESS;
}
