#include "game.h"
#include "piece.h"

#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>

#define NB_PIECES 4

/**
 Test si expected == value. Si les deux entiers sont égaux, renvoie vrai, sinon affiche un message d'erreur dans la sortie standard d'erreur et renvoie faux.
 */
bool test_equality_int(int expected, int value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}

/**
 Test si expected == value. Si les deux booléens sont égaux, renvoie vrai, sinon affiche un message d'erreur dans la sortie standard d'erreur et renvoie faux.
 */
bool test_equality_bool(bool expected, bool value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}


piece Tab_de_pieces[NB_PIECES];

/**
 * Initialise le tableau de pièce qui va servir à créer la game GAME_TO_TEST.
 **/
 
void set_up() {
  Tab_de_pieces[0] = new_piece_rh(0, 3, true, true);
  Tab_de_pieces[1] = new_piece_rh(3, 3, true, false);
  Tab_de_pieces[2] = new_piece_rh(4, 1, true, true);
  Tab_de_pieces[3] = new_piece_rh(2, 1, false, false);
}

/**
 * Créer la game GAME_TO_TEST.
 **/

game set_game(){
  set_up();
	return new_game_hr(4, Tab_de_pieces);
}

/**
 * Toutes nos fonctions de Tests.
 **/	
	
bool test_new_game(){
	game g=set_game();
	bool result=true;
	result=test_equality_int(4,game_nb_pieces(g),"new_game_hr : nb_pieces");
	for(int i=0; i<NB_PIECES; i++){
		result = result && test_equality_int(get_x(Tab_de_pieces[i]), get_x(game_piece(g,i)),"new_game_hr : get_x");
		result = result && test_equality_int(get_y(Tab_de_pieces[i]), get_y(game_piece(g,i)),"new_game_hr : get_y");
		if(can_move_x(Tab_de_pieces[i])){
			result = result && test_equality_int(1, get_height(Tab_de_pieces[i]), "new_game_hr : can_move_x");
          }
          else {
			  result = result && test_equality_bool(true, can_move_y(game_piece(g,i)), "new_game_hr : can_move_y");
			  result = result && test_equality_int(1, get_width(Tab_de_pieces[i]), "new_game_hr : get_width");
          }
	  }
	  result=result&&test_equality_int(0,game_nb_moves(g),"new_game_hr : mouvement");
	  result=result&&test_equality_int(6,game_width(g),"new_game_hr : width");
	  result=result&&test_equality_int(6,game_height(g),"new_game_hr : height");
	  delete_game(g);
	  return result;
}



/*bool test_delete_game(){
	game g=set_game();
	delete_game(g);
	//Lancer Valgrind pour vérifier qu'on a free g, free le tableau de g et free les pieces du tableau de g
	return true;
}*/


bool test_copy_game(){
	game game_src=set_game();
	bool result=true;
	piece tab[1];
	tab[0]=new_piece_rh(0,0,true,true);
	game game_dst=new_game(6,6,1,tab);
	copy_game(game_src,game_dst);
	result=test_equality_int(game_nb_pieces(game_src),game_nb_pieces(game_dst), "copy_game : nb_pieces");
	for(int i=0;i<game_nb_pieces(game_src);++i){
		result = result && test_equality_int(get_x(game_piece(game_src,i)), get_x(game_piece(game_dst,i)), "copy get_x");
		result = result && test_equality_int(get_y(game_piece(game_src,i)), get_y(game_piece(game_dst,i)), "copy get_y");
		result = result && test_equality_int(get_width(game_piece(game_src,i)), get_width(game_piece(game_dst,i)), "copy get_width");
		result = result && test_equality_int(get_height(game_piece(game_src,i)), get_height(game_piece(game_dst,i)), "copy get_height");
	}
	result=result && test_equality_int(game_nb_moves(game_src),game_nb_moves(game_dst), "copy_game : nb_mouvement");
	result=result&&test_equality_int(game_width(game_src),game_width(game_dst),"copy_game : width");
	result=result&&test_equality_int(game_height(game_src),game_height(game_dst),"copy_game : height");
	delete_game(game_src);
	delete_game(game_dst);
	return result;
	
}

 bool test_game_over(){
   bool result=true;
   game g=set_game();
   play_move(g,3,DOWN,1);
   play_move(g,1,DOWN,3);
   play_move(g,0,RIGHT,4);
   result=result && test_equality_bool(true,game_over_hr(g), "game over : game_over_hr");
   result=result && test_equality_bool(true,game_over(g,4,3), "game over : game_over");
   delete_game(g);
   return result;
   
 }

 bool test_play_move(){
   bool result=true;
   game g=set_game();//cas sans intersection
   play_move(g,3,DOWN,1);
   result=result && test_equality_int(2,get_x(game_piece(g,3)),"play move : sans intersection, abscisse");
   result=result && test_equality_int(0,get_y(game_piece(g,3)),"play move : sans intersection, ordonnée");
   game g1=set_game();//cas avec intersection
   //play_move(g1,3,DOWN,1);
   play_move(g1,0,RIGHT,4);
   result=result && test_equality_int(0,get_x(game_piece(g1,0)),"play move : avec intersection, abscisse");
   result=result && test_equality_int(3,get_y(game_piece(g1,0)),"play move : avec intersection, ordonnée");
   delete_game(g);
   return result;
 }

bool test_game_nb_moves(){
  bool result=true;
  game g=set_game();//cas sans intersection
  play_move(g,3,DOWN,1);
  play_move(g,1,DOWN,3);
  play_move(g,0,RIGHT,4);
  result=result && test_equality_int(3,game_nb_moves(g),"nombre de mouvement");
  game g1=set_game();//cas avec intersection
  play_move(g1,3,DOWN,1);
  play_move(g1,0,RIGHT,4);
  play_move(g1,1,DOWN,3);
  result=result && test_equality_int(2,game_nb_moves(g1),"nombre de mouvement");
  delete_game(g);
  delete_game(g1);
  return result;
}

bool test_game_square_piece(){
  bool result=true;
  game g=set_game();
  result=result && test_equality_int(1,game_square_piece(g,3,3),"game_square_piece");
  result=result && test_equality_int(1,game_square_piece(g,3,4),"game_square_piece");
  result=result && test_equality_int(0,game_square_piece(g,0,3),"game_square_piece");
  result=result && test_equality_int(-1,game_square_piece(g,0,0),"game_square_piece" );
  delete_game(g);
  return result;
}



int main (int argc, char *argv[])
{
  bool result= true;
  
  result = result && test_equality_bool(true, test_new_game(), "new_game");
  if (result)
    printf("Test new_game: OK \n");
    
  /*result = result && test_equality_bool(true, test_delete_game(), "delete_game");
  if (result)
    printf("Test delete_game: OK \n");*/
    
  result = result && test_equality_bool(true, test_copy_game(), "copy_game");
  if (result)
    printf("Test copy_game: OK\n");
  result = result && test_equality_bool(true, test_game_over(), "game_over");
  if (result)
    printf("Test game_over: OK\n");
  result = result && test_equality_bool(true, test_play_move(), "play_move");
  if (result)
    printf("Test play_move: OK\n");
  result = result && test_equality_bool(true, test_game_nb_moves(), "nb_moves");
  if (result)
    printf("Test nb_moves: OK\n");
  result = result && test_equality_bool(true, test_game_square_piece(), "game_square_piece");
  if (result)
    printf("Test game_square_piece: OK\n\n");

  if (result) {
    printf("Test game: OK \n");
    return EXIT_SUCCESS;
  }
  else
    return EXIT_FAILURE;
}
  
  









	



